@@ -2,16 +2,17 @@ from functools import partial
 import json
 from pydantic import BaseModel
 from pydantic.json import custom_pydantic_encoder, pydantic_encoder
-from typing import Any, cast, Dict
+from typing import Any, cast, Dict, Union
 
+from fastmicro.messaging.header import T, HeaderABC
 from fastmicro.serializer import SerializerABC
 
 
 class Serializer(SerializerABC):
     @staticmethod
-    async def serialize(data: BaseModel) -> bytes:
+    async def serialize(data: Union[BaseModel, HeaderABC[T]]) -> bytes:
         if (
-            hasattr(data, "message")
+            isinstance(data, HeaderABC)
             and isinstance(data.message, BaseModel)
             and data.message.__config__.json_encoders
         ):
